@@ -3,19 +3,19 @@
 /* build batch file */
 /*
 #ifdef APE_BATCH_FILE_VERSION
-Set _MA=1016
-Set _MAV=10.16
+Set _MA=1157
+Set _MAV=1157
 #endif
 */
 
 /* major version number */
-#define MAC_VERSION_MAJOR 10
+#define APE_VERSION_MAJOR 11
 
 /* build version number */
-#define MAC_VERSION_REVISION 16
-#define MAC_VERSION_REVISION_NUMBER 16
+#define APE_VERSION_REVISION 57
+#define APE_VERSION_REVISION_NUMBER 57 // needed because a number like 08 is interpreted as octal
 
 /* library interface version, update this whenever the signature of an exported function changes */
-#define MAC_INTERFACE_VERSION 10
+#define APE_INTERFACE_VERSION 14
 
 /* leave this so the end of file doesn't get truncated */
