@@ -1,6 +1,3 @@
-// MAC.cpp : Defines the class behaviors for the application.
-//
-
 #include "stdafx.h"
 #include "MAC.h"
 #include "MACDlg.h"
@@ -24,7 +21,9 @@ CMACApp::CMACApp()
     DWORD dwLastError = GetLastError();
     m_bAnotherInstanceRunning = (dwLastError == ERROR_ALREADY_EXISTS);
     m_pMACDlg = APE_NULL;
-
+    m_nMonitorDPI = 0;
+    m_nFontHeight = 0;
+    
     // start GDI plus
     Gdiplus::GdiplusStartupInput gdiplusStartupInput; // filled in by the constructor
     ULONG_PTR gdiplusToken = 0;
@@ -233,6 +232,7 @@ bool CMACApp::SetScale(double dScale)
     if (fabs(dScale - m_dScale) > 0.01)
     {
         m_dScale = dScale;
+        LoadFont();
         bResult = true;
     }
     return bResult;
@@ -263,3 +263,50 @@ Gdiplus::Bitmap * CMACApp::GetMonkeyImage()
     }
     return m_spbmpMonkey;
 }
+
+void CMACApp::LoadFont()
+{
+    // reset if we already exist
+    if (m_Font.GetSafeHandle() != APE_NULL)
+        m_Font.DeleteObject();
+
+    // get the system font using SPI_GETNONCLIENTMETRICS
+    NONCLIENTMETRICS ncm;
+    APE_CLEAR(ncm);
+    ncm.cbSize = sizeof(ncm);
+
+    // try to get DPI aware
+    BOOL bResult = false;
+    HMODULE hUser32 = LoadLibrary(_T("user32.dll"));
+    if (hUser32 != APE_NULL)
+    {
+        BOOL(STDAPICALLTYPE * pSystemParametersInfoForDpi) (IN UINT uiAction, IN UINT uiParam, PVOID pvParam, IN UINT fWinIni, IN UINT dpi);
+        *(reinterpret_cast<FARPROC*>(&pSystemParametersInfoForDpi)) = GetProcAddress(hUser32, "SystemParametersInfoForDpi");
+        if (pSystemParametersInfoForDpi != APE_NULL)
+        {
+            bResult = pSystemParametersInfoForDpi(SPI_GETNONCLIENTMETRICS, ncm.cbSize, &ncm, 0, m_nMonitorDPI);
+        }
+
+        FreeLibrary(hUser32);
+    }
+
+    // get non-DPI aware
+    if (bResult == false)
+    {
+        // get font
+        SystemParametersInfo(SPI_GETNONCLIENTMETRICS, ncm.cbSize, &ncm, 0);
+
+        // scale by scale factor (since we're not DPI aware)
+        ncm.lfMessageFont.lfHeight = GetSize(ncm.lfMessageFont.lfHeight);
+    }
+
+    // scale by the setting
+    double dSize = static_cast<double>(GetSettings()->LoadSetting(_T("Font Size"), 100));
+    ncm.lfMessageFont.lfHeight = static_cast<LONG>((static_cast<double>(ncm.lfMessageFont.lfHeight) * dSize) / 100.0);
+
+    // build font
+    m_Font.CreateFontIndirect(&ncm.lfMessageFont);
+
+    // store the font height
+    m_nFontHeight = -ncm.lfMessageFont.lfHeight;
+}
